
from typing import Any, Dict, Union


Property = Union[str, int, float, bool]
Properties = Dict[str, Property]
PropertyDefinition = Dict[str, Dict[str, Any]]
PropertyDefinitions = Dict[str, PropertyDefinition]
