
from setuptools import setup, find_namespace_packages


setup(
    name='qtoggleserver-mppsolar',
    version='1.4.5',
    description='MPP Solar inverters support for qToggleServer',
    author='Calin Crisan',
    author_email='ccrisan@gmail.com',
    license='Apache 2.0',

    packages=find_namespace_packages()
)
